﻿using System;
using System.Windows.Forms;
using System.IO.Ports;
using System.Text;
using System.IO;

namespace SerialSample13
{
    public partial class Form1 : Form
    {
        SerialPort ComPort = new SerialPort("COM1");
        const int DMA_BLOCK_LENGTH = 256;
        string filePath = "";
        uint crcSum;
        public Form1()
        {
            InitializeComponent();
            ComPort.BaudRate = 115200;
            ComPort.Parity = Parity.None;
            ComPort.StopBits = StopBits.One;
            ComPort.DataBits = 8;
            ComPort.Handshake = Handshake.None;
            ComPort.DataReceived += OnSerialDataReceived;
            ComPort.Open();
        }

        private void OnSerialDataReceived(object sender, SerialDataReceivedEventArgs args)
        {
            crcSum = 0;
            for (int i = 0; i < 4; i++)
            {
                crcSum >>= 8;
                crcSum |= (uint)(ComPort.ReadByte() << 24);
            }
            lblCRC.Invoke(new EventHandler(lblCRC_DataReceived));
        }

        private void lblCRC_DataReceived(object sender, EventArgs e)
        {
            lblCRC.Text = crcSum.ToString("X8");
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[DMA_BLOCK_LENGTH];
            byte[] strBytes = Encoding.ASCII.GetBytes(textBox1.Text);
            byte[] lengthBytes = BitConverter.GetBytes(strBytes.Length);
            for (int i = 0; i < 4; i++) buffer[i] = lengthBytes[i];
            Array.Copy(strBytes, 0, buffer, 4, strBytes.Length);
            ComPort.Write(buffer, 0, DMA_BLOCK_LENGTH);
        }

        private void btnSendFile_Click(object sender, EventArgs e)
        {
            sendFile();
        }

        private void sendFile()
        {
            if (filePath == "") return;
            byte[] buffer = new byte[DMA_BLOCK_LENGTH];
            using (BinaryReader b = new BinaryReader(File.Open(filePath, FileMode.Open)))
            {
                int fileLength = (int)b.BaseStream.Length;
                for (int i = 0; i < fileLength; )
                {
                    int readBytes;
                    if (i == 0)
                    {
                        byte[] lengthBytes = BitConverter.GetBytes(fileLength);
                        for (int j = 0; j < 4; j++) buffer[j] = lengthBytes[j];
                        readBytes = b.Read(buffer, 4, DMA_BLOCK_LENGTH - 4);
                        ComPort.Write(buffer, 0, DMA_BLOCK_LENGTH);
                    }
                    else
                    {
                        readBytes = b.Read(buffer, 0, DMA_BLOCK_LENGTH);
                        ComPort.Write(buffer, 0, DMA_BLOCK_LENGTH);
                    }
                    i += readBytes;
                    progressBar1.Value = i * 100 / fileLength;
                }
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            DialogResult result = openFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                filePath = openFileDialog.FileName;
                lblPath.Text = "path: " + filePath;
            }

        }

    }
}
