﻿using System;
using System.IO.Ports;
using System.Windows.Forms;

namespace SerialSample7
{
    public partial class Form1 : Form
    {
        SerialPort ComPort = new SerialPort("COM1");
        
        byte[] segmentSets = new byte[]{0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F };
        public Form1()
        {
            InitializeComponent();
            if (ComPort.IsOpen) ComPort.Close();
            ComPort.BaudRate = 9600;
            ComPort.Parity = Parity.None;
            ComPort.StopBits = StopBits.One;
            ComPort.DataBits = 8;
            ComPort.Handshake = Handshake.None;
            ComPort.Open();
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            byte[] digits = new byte[4];
            stringToSegments(txtDigits.Text, digits);
            ComPort.Write(digits, 0, 4);
        }

        private void stringToSegments(String str, byte[] digits)
        {
            bool dot = false;
            int j = 0;
            for (int i = 0; i < 4; i++) digits[i] = 0;
            for (int i = str.Length - 1; i >= 0; i--)
            {
                char c = str[i];
                if (c == '.' && dot)
                {
                    digits[j] = 0x80;
                    j++;
                    continue;
                }
                if (c == '.') dot = true;
                if (Char.IsDigit(c))
                {                   
                    digits[j] = segmentSets[c - '0'];
                    if (dot) digits[j] += 0x80;
                    dot = false;
                    j++;
                    continue;
                }
            }
            if (dot) digits[j] = 0x80;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            byte[] digits = new byte[4];
            string dot = "";
            if (DateTime.Now.Millisecond > 500) dot = ".";
            string time = DateTime.Now.Hour.ToString("00") + dot + DateTime.Now.Minute.ToString("00");
            stringToSegments(time, digits);
            ComPort.Write(digits, 0, 4);

            if (chkUpdate.Checked)
            {
                txtDigits.Text = time;
                string timeHex = digits[3].ToString("X2") + digits[2].ToString("X2") + digits[1].ToString("X2") + digits[0].ToString("X2");
                maskedTextBox1.Text = timeHex;
            }
        }

        private void chkTime_CheckedChanged(object sender, EventArgs e)
        {
            timer1.Enabled = chkTime.Checked;
        }

        private void btnHex_Click(object sender, EventArgs e)
        {
            byte[] digits = new byte[4];
            string[] digitsStr = maskedTextBox1.Text.Split(new char[] {' '});
            try
            {
                for (int i = 0; i < 4; i++) digits[3 - i] = Convert.ToByte(digitsStr[i], 16);
                ComPort.Write(digits, 0, 4);
            }
            catch (System.FormatException ex)
            {
            }
            
        }
    }
}
